#!/usr/bin/env node
/**
 * Copyright (c) 2015-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

'use strict';

// Makes the script crash on unhandled rejections instead of silently
// ignoring them. In the future, promise rejections that are not handled will
// terminate the Node.js process with a non-zero exit code.
process.on('unhandledRejection', err => {
  throw err;
});

const spawn = require('react-dev-utils/crossSpawn');
const chalk = require('chalk');
const xl = chalk.cyan('[Express Labs]') + ': ';
const complete = chalk.green('Complete');
const args = process.argv.slice(2);

function logXL(message) {
  console.log(xl + message);
}

function init(string, log = false) {
  const breakup = string.split(' ');
  const result = spawn.sync(breakup[0], breakup.slice(1), {
    stdio: 'inherit',
  });

  if (log) {
    console.log(result);
  }

  return result;
}

function checkResult(result) {
  if (result.signal) {
    if (result.signal === 'SIGKILL') {
      console.log(
        'The build failed because the process exited too early. ' +
          'This probably means the system ran out of memory or someone called ' +
          '`kill -9` on the process.'
      );
    } else if (result.signal === 'SIGTERM') {
      console.log(
        'The build failed because the process exited too early. ' +
          'Someone might have called `kill` or `killall`, or the system could ' +
          'be shutting down.'
      );
    }

    process.exit(1);
  }
}

const scriptIndex = args.findIndex(
  x => x === 'build' || x === 'eject' || x === 'start' || x === 'test'
);
const script = scriptIndex === -1 ? args[0] : args[scriptIndex];
const nodeArgs = scriptIndex > 0 ? args.slice(0, scriptIndex) : [];

switch (script) {
  case 'build':
  case 'eject':
  case 'start':
  case 'test': {
    const result = spawn.sync(
      'node',
      nodeArgs.concat(require.resolve('../scripts/' + script)).concat(args.slice(scriptIndex + 1)),
      { stdio: 'inherit' }
    );
    checkResult(result);

    process.exit(result.status);
    break;
  }
  case 'lint': {
    logXL('Linting JS...');
    init('eslint . --fix --ext .js --ext .jsx');
    logXL(`JS Lint - ${complete}\n`);

    logXL('Linting Styles...');
    init('stylelint --fix src/**/*.scss');
    logXL(`Styles Lint - ${complete}\n`);
    break;
  }
  case 'travis:lint': {
    logXL('Linting JS...');
    init('eslint . --ext .js --ext .jsx');
    logXL('JS Lint - Complete.');

    logXL('Linting Styles...');
    init('stylelint src/**/*.scss');
    logXL('Styles Lint - Complete.');
    break;
  }
  case 'travis:test': {
    const result = init(`node ${require.resolve('../scripts/test.js')} -w=4 --silent --coverage`);
    checkResult(result);

    process.exit(result.status);
    break;
  }
  default:
    console.log('Unknown script "' + script + '".');
    console.log('Perhaps you need to update react-scripts?');
    console.log('See: https://facebook.github.io/create-react-app/docs/updating-to-new-releases');
    break;
}
