import React from 'react';
import { mount } from 'enzyme';
import { MockedProvider as MockedApolloProvider } from 'react-apollo/test-utils';
import errorBoundaryFactory, { ErrorBoundary } from '../errorBoundary';

describe('the errorboundary higher order component', () => {
  describe('that extends React.Component called', () => {
    describe('ErrorBoundary', () => {
      const getTestProps = () => ({
        component: jest.fn(),
        logClientError: jest.fn(),
        priority: '1',
      });
      const testProps = getTestProps();
      let wrapper;

      beforeEach(() => {
        wrapper = new ErrorBoundary(testProps);
      });

      describe('constructor', () => {
        it('hasError should be set to false', () => {
          expect(wrapper.state.hasError).toBe(false);
        });
      });

      describe('componentDidCatch', () => {
        beforeEach(() => {
          const testComponent = jest.fn();
          wrapper = errorBoundaryFactory()(testComponent);
          wrapper.state = {};
          wrapper.children = () => {
            throw new Error();
          };
          wrapper.componentDidCatch = jest.fn((() => {
            wrapper.state.hasError = true;
          }));
        });

        it('caught component', () => {
          mount(
            <MockedApolloProvider>
              {wrapper()}
            </MockedApolloProvider>
          );
          wrapper.componentDidCatch();

          expect(wrapper.children).toThrow();
          expect(wrapper.componentDidCatch).toBeCalled();
          expect(wrapper.state.hasError).toBe(true);
        });

        it('invoked trackJs when found', () => {
          global.window.trackJs = {
            console: {
              error: jest.fn(),
            },
          };
          mount(
            <MockedApolloProvider>
              {wrapper()}
            </MockedApolloProvider>
          );
          wrapper.componentDidCatch();

          expect(global.window.trackJs.console.error).toBeCalled();
        });
      });
    });
  });

  describe('contains a function called', () => {
    const testComponent = jest.fn();
    const testConfig = {
      priority: '1',
      errorView: 'Hi there :)',
    };

    describe('errorBoundaryFactory, where when called', () => {
      it('returns a function that passes the wrapped component and a config to a higher order function', () => {
        expect(errorBoundaryFactory(testConfig)(testComponent)).toEqual(expect.any(Function));
        expect(errorBoundaryFactory(testConfig)(testComponent).displayName).toBe('errorBoundary');
      });

      describe('the higher order function that this function returns', () => {
        const hoc = errorBoundaryFactory(testConfig)(testComponent)({ fakeProps: true });

        it('should pass the arguments as props to the errorBoundary component', () => {
          expect(hoc.props).toEqual({
            component: testComponent,
            fakeProps: true,
            priority: testConfig.priority,
            errorView: 'Hi there :)',
          });
        });
      });
    });
  });
});
