# mapProps()

The mapProps higher order component takes supplied props and maps them to different props. This HOC allows you to decouple the wrapped component from the shape of an incoming prop. This makes the component more portable, because it is no longer tied to, for example, the shape of a fetched async response. It is also possible to completely flatten the shape of the props supplied to your component. This makes your component "dumber", possibly stateless, and at the very least a good candidate for a transition to React.PureComponent.  

For example, lets say there is an complicated incoming prop like `some.very.long.object.path.firstName`. Using this higher order component, you can map that very long path to just `firstName`.

Hint: Place the debugPropStream HOC before this HOC to print in console all the props mapProps is able to access.

## Pseudocode Example
```javascript
// In your component's index.js file...
import { getPath } from '@express-labs/raven-tools';
import { mapProps } from '../hocs';
import HelloWorld from './HelloWorld';

export mapProps({
  options: {
    mode: 'merge', // modes are "merge" (default), and "gate".
    maps: {
      firstName: ownProps => getPath(ownProps, 'some.very.long.object.path.firstName', 'Hello!');    
    },
  },
})(HelloWorld);
```

## Arguments
mapProps requires a single argument that is single javascript object.  The object can contain the following properties: mode, maps, mapFunction/

### mode
Valid values for mode are "merge" and "gate".

| name | description |
|------|-------------|
| merge  | This is the default. Any props generated by mapProps will overwrite existing props with the same name. |
| gate | Only props generated by mapProps are applied to the wrapped component.  All other props are discarded. |

### maps
The maps property must be an object where object property names become the names of new props.  Each property in the mappings object must be a function that returns a value.  The function is supplied with the current props as the only argument.  Common convention is to call this argument "ownProps", but you can name it anything, really.  Example of maps.

Example of a maps object:

```javascript
{
  maps: {
    capitalName: ownProps => ownProps.user.confirmed.name.toUpperCase(),
    addressLine1: ownProps => ownProps.user.confirmed.address1.toLowerCase(),
  }
}
```

In the above example, two new props are created: "capitalName", and "addressLine1".

### mapFunction
The mapFunction property must be a function that returns a new object.  The returned object is passed to the target component as props. The function is supplied with the current props as the only argument. Common convention is to call this argument "ownProps", but you can name it anything, really.  

Example of mapfunction:

```javascript
{
  mapFunction: (ownProps) => {
    const { blah, sneh, flarg } = ownProps.data.thing;
    const ploo = `${sneh} - ${flarg}`;
    return ({
      blah: blah.toUpperCase(),
      ploo,
    })
  }
}
```
