import React from 'react';
import { shallow } from 'enzyme';
import mapPropsFactory, { MapProps } from '../mapProps';

describe('The mapProps higher order component', () => {
  describe('that extends React.Component called', () => {
    describe('MapProps', () => {
      const getTestProps = () => ({
        component: jest.fn(),
        mapPropsMaps: {
          prop4: jest.fn(),
          prop5: jest.fn(),
          prop6: jest.fn()
        },
        prop1: 'foo',
        prop2: 'bar',
        prop3: 'bazz'
      });
      const testProps = getTestProps();
      const mapFunctionMockResult = { aProp: 'value', prop3: 'override' };
      let wrapper;

      beforeEach(() => {
        wrapper = shallow(<MapProps {...testProps} />);
      });

      describe('that has two settings:', () => {
        beforeAll(() => {
          testProps.mapFunction = jest.fn().mockReturnValue(mapFunctionMockResult);
        });

        afterAll(() => {
          delete testProps.mapFunction;
        });

        describe('MERGE, which', () => {
          it('should combine the passed-in props, overwriting duplicate properties, and pass them as props to the wrapped component', () => {
            const { prop1, prop2, prop3 } = testProps;
            const expectedProps = Object.assign(
              {},
              { prop1 },
              { prop2 },
              { prop3 },
              mapFunctionMockResult
            );
            expect(wrapper.props()).toEqual(expectedProps);
          });
        });

        describe('GATE, which', () => {
          beforeAll(() => {
            testProps.mapPropsMode = 'gate';
          });

          afterAll(() => {
            delete testProps.mapFunction;
            delete testProps.mapPropsMode;
          });

          it('should only allow the modified props to passed to the wrapped component', () => {
            expect(wrapper.props()).toEqual(Object.assign({}, mapFunctionMockResult));
          });
        });
      });

      describe('allows for modifying props before they are passed to the wrapped component. If a "mapFunction"', () => {
        describe('IS PASSED IN', () => {
          beforeAll(() => {
            testProps.mapFunction = jest.fn().mockReturnValue(mapFunctionMockResult);
          });

          afterAll(() => {
            delete testProps.mapFunction;
          });

          it('should include the returned object as props passed to the wrapped component', () => {
            expect(wrapper.props()).toEqual(expect.objectContaining(mapFunctionMockResult));
          });
        });

        describe('is NOT PASSED IN', () => {
          const { prop1, prop2, prop3 } = testProps;

          beforeEach(() => {
            testProps.mapPropsMaps.prop4 = jest.fn().mockReturnValueOnce({});
            testProps.mapPropsMaps.prop5 = jest.fn().mockReturnValueOnce({});
            testProps.mapPropsMaps.prop6 = jest.fn().mockReturnValueOnce({});
          });

          it('should include the returned object as props passed to the wrapped component', () => {
            expect(wrapper.props()).toEqual(expect.objectContaining({ prop1, prop2, prop3 }));
          });

          it('should include the mapPropsMaps object as keys to the new props object', () => {
            expect(wrapper.props()).toEqual(
              expect.objectContaining({
                prop4: expect.any(Object),
                prop5: expect.any(Object),
                prop6: expect.any(Object)
              })
            );
          });
        });
      });
    });
  });

  describe('contains a function called', () => {
    const testComponent = jest.fn();
    const testConfig = {
      mapFunction: jest.fn,
      maps: {},
      mode: 'merge'
    };

    describe('mapPropsFactory, where when called', () => {
      it('returns a function that passes the wrapped component and a config to a higher order function', () => {
        expect(mapPropsFactory(testConfig)(testComponent)).toEqual(expect.any(Function));
        expect(mapPropsFactory(testConfig)(testComponent).displayName).toBe('mapProps');
      });

      describe('the higher order function that this function returns', () => {
        const hoc = mapPropsFactory(testConfig)(testComponent)({ fakeProps: true });

        it('should pass the arguments as props to the MapProps component', () => {
          expect(hoc.props).toEqual({
            component: testComponent,
            fakeProps: true,
            mapPropsMaps: testConfig.maps,
            mapPropsMode: testConfig.mode,
            mapFunction: testConfig.mapFunction
          });
        });
      });
    });
  });
});
