# withPageLoading

The withPageLoading higher order component decouples page-level loading and error states from a page container. It automatically searches for properties inserted by graphql.  You can also supply an array of custom tests to search for other properties that indicate an async loading state.

Using withPageLoading on a page-level container eliminates the need developers to replicate code desinged to detect loading and display page skeletons.

The wrapped page container will only render after all the graphql queries (and any other tests you supply) for that page are successfully loaded. This means that your wrapped container...

- has less need to check for undefined props. (e.g.  `data && data.something && data.something.value`)
- does not need to insert Skeleton, Error400, Error500
- does not need to call removeSkeleton()
- Reduces complicated logic needed to notify Adobe target tests that the component has mounted and has data.

## Pseudocode Example

```javascript
import { compose, graphql } from 'react-apollo';
import { withPageLoading } from '../hocs';
import { getSomeData } from '../../queries';

export default compose(
  withPageLoading({
    custom: [
      {
        errorData: ownProps => ownProps.blarg.getErrorData(),
        isError: ownProps => ownProps.foo.error,
        isLoading: ownProps => ownProps.sneh.checkIfLoading(),
        isSuccess: ownProps => ownProps.ploo.checkIfSuccess(),
      }
    ]
  }),
  graphql(getSomeData, { name: 'whatever' }),
);
```

## Arguments
withPageLoading accepts a single argument that is an object.  The object may have an optional "custom" property with is an array of objects. By default, withPageLoading scans for all graphql calls.  You only need to add objects to this an array to create custom tests.

Objects added to the "custom" array must have the following shape and all object properties are required.

| name | type | description | return value |
| ---- | ---- | ----------- | ------------ |
| errorData | function | parses the supplied props and returns an arbitrary value that the wrapped component. The wrapped component must decide what to do with the data. A possible use for this option is to compute an error message that is passed to the wrapped component. | any |
| isError | function | This function is passed the current props and returns true for an error or false if no error | boolean |
| isLoading | function | This function is passed the current props and returns true if still loading or false if finished loading | boolean |
| isSuccess | function | This function is passed the current props and returns true if the async call was a success, false if not. | boolean |
