import React from 'react';
import { compose } from 'react-apollo';
import { mount, render, shallow } from 'enzyme';
import withPageLoading, { TestResult, WithPageLoading } from '../withPageLoading';

describe('class TestResult', () => {
  describe('get()', () => {
    it('Should return a default TestResult object when new is called with no constructor args', () => {
      const instance = new TestResult();
      expect(instance.get()).toEqual({
        errorData: null,
        isError: null,
        isLoading: null,
        isSuccess: null
      });
    });
  });
});

describe('withInventory HOC', () => {
  describe('isFunction()', () => {
    it('should return false if not a function', () => {
      expect(WithPageLoading.isFunction(false)).toBe(false);
    });
    it('should return true if it is a function', () => {
      expect(WithPageLoading.isFunction(() => {})).toBe(true);
    });
  });
  describe('isPropApolloQuery()', () => {
    it('should return true if the prop has both the loading and network status properties', () => {
      expect(
        WithPageLoading.isPropApolloQuery({
          loading: true,
          networkStatus: 400
        })
      ).toBe(true);
    });
    it('should return false if the prop does not have a loading property', () => {
      expect(
        WithPageLoading.isPropApolloQuery({
          networkStatus: 400
        })
      ).toBe(false);
    });
    it('should return false if the prop does not have a networkStatus property', () => {
      expect(
        WithPageLoading.isPropApolloQuery({
          loading: true
        })
      ).toBe(false);
    });
  });
  describe('scanForApolloPropNames()', () => {
    it('should return the name of props it thinks were added by Apollo graphql() hoc', () => {
      // For more information on networkStatus, see
      // https://github.com/apollographql/apollo-client/blob/master/packages/apollo-client/src/core/networkStatus.ts
      const props = {
        noApollo: { blarg: true },
        yesApollo: {
          loading: false,
          networkStatus: 7
        },
        yesApolloAlso: {
          loading: true,
          networkStatus: 1
        }
      };
      expect(WithPageLoading.scanForApolloPropNames(props)).toEqual(['yesApollo', 'yesApolloAlso']);
    });
  });
  describe('createApolloTestResults()', () => {
    let ownProps;

    beforeEach(() => {
      ownProps = {
        noApollo: {
          blarg: true
        },
        apolloSuccess: {
          loading: false,
          networkStatus: 7
        },
        apolloLoading: {
          loading: true,
          networkStatus: 1
        },
        apolloWithError: {
          loading: false,
          networkStatus: 8,
          error: 'I am error data'
        }
      };
    });
    it('should set errorData to true if there was an error in a graphql query', () => {
      expect(WithPageLoading.createApolloTestResults(['apolloWithError'], ownProps)).toEqual([
        {
          errorData: 'I am error data',
          isError: true,
          isLoading: false,
          isSuccess: false
        }
      ]);
    });
    it('should set isSuccess to true if a graphql query had no errors', () => {
      expect(WithPageLoading.createApolloTestResults(['apolloSuccess'], ownProps)).toEqual([
        {
          errorData: false,
          isError: false,
          isLoading: false,
          isSuccess: true
        }
      ]);
    });
    it('should set isLoading to true if a graphql query is loading', () => {
      expect(WithPageLoading.createApolloTestResults(['apolloLoading'], ownProps)).toEqual([
        {
          errorData: false,
          isError: false,
          isLoading: true,
          isSuccess: false
        }
      ]);
    });
  });
  describe('createCustomTestResults()', () => {
    it('should run custom tests and return an array of test objects', () => {
      const props = {
        getFlavor: {
          loading: false,
          flavor: 'snozzberry'
        },
        withPageLoadingOptions: {
          custom: [
            {
              errorData: ownProps => ownProps.getFlavor.error || false,
              isError: ownProps => !!ownProps.getFlavor.error,
              isLoading: ownProps => !!ownProps.getFlavor.loading,
              isSuccess: ownProps => !ownProps.getFlavor.loading && typeof ownProps.getFlavor.flavor === 'string'
            }
          ]
        }
      };
      expect(WithPageLoading.createCustomTestResults(props)).toEqual([
        {
          errorData: false,
          isError: false,
          isLoading: false,
          isSuccess: true
        }
      ]);
    });
  });
  describe('isLoading()', () => {
    it('should return true if at least one test in the array of test results says that it is still loading', () => {
      const testResults = [
        new TestResult({ isLoading: false }),
        new TestResult({ isLoading: false }),
        new TestResult({ isLoading: true })
      ];
      expect(WithPageLoading.isLoading(testResults)).toBe(true);
    });
    it('should return false if not even one test in the array of test results says that it is still loading', () => {
      const testResults = [
        new TestResult({ isLoading: false }),
        new TestResult({ isLoading: false }),
        new TestResult({ isLoading: false })
      ];
      expect(WithPageLoading.isLoading(testResults)).toBe(false);
    });
  });
  describe('isError()', () => {
    it('should return true if at least one test in the array of test results says that it is an error', () => {
      const testResults = [
        new TestResult({ isError: false }),
        new TestResult({ isError: false }),
        new TestResult({ isError: true })
      ];
      expect(WithPageLoading.isError(testResults)).toBe(true);
    });
    it('should return false if not even one test in the array of test results says that it is an error', () => {
      const testResults = [
        new TestResult({ isError: false }),
        new TestResult({ isError: false }),
        new TestResult({ isError: false })
      ];
      expect(WithPageLoading.isError(testResults)).toBe(false);
    });
  });
  describe('makeErrorDataArray', () => {
    it('should return an array of all the errorData values if isError is true and errorData has a value', () => {
      const testResults = [
        new TestResult({ isError: true, errorData: 'A' }),
        new TestResult({ isError: true, errorData: 'B' }),
        new TestResult({ isError: true, errorData: 'C' })
      ];
      expect(WithPageLoading.makeErrorDataArray(testResults)).toEqual(['A', 'B', 'C']);
    });
    it('should return an empty array if isError is true but errorData DOES NOT have a value', () => {
      const testResults = [
        new TestResult({ isError: true }),
        new TestResult({ isError: true }),
        new TestResult({ isError: true })
      ];
      expect(WithPageLoading.makeErrorDataArray(testResults)).toEqual([]);
    });
    it('shour return an empty array if errorData values are set but isError is false', () => {
      const testResults = [
        new TestResult({ isError: false, errorData: 'A' }),
        new TestResult({ isError: false, errorData: 'B' }),
        new TestResult({ isError: false, errorData: 'C' })
      ];
      expect(WithPageLoading.makeErrorDataArray(testResults)).toEqual([]);
    });
  });
  describe('constructor()', () => {
    it('should set the state to a default loading state as defined by defaultState', () => {
      const instance = new WithPageLoading();
      expect(instance.state).toEqual(WithPageLoading.defaultState);
      expect(instance.state).toEqual({
        isLoading: true,
        isError: null,
        errorData: null
      });
    });
  });
  describe('getDerivedStateFromProps()', () => {
    it('should run all apollo tests, and custom tests, and set isLoading, isError, and errorData accordingly', () => {
      const propsWithEverything = {
        getFlavor: {
          loading: true,
          networkStatus: 1
        },
        getSprinkles: {
          loading: false,
          networkStatus: 8,
          error: 'totally an error, brah.'
        },
        customCone: {
          iBeLoading: 'nope',
          coneBabyCone: 'cup'
        },
        withPageLoadingOptions: {
          custom: [
            {
              errorData: () => false,
              isError: () => false,
              isLoading: ownProps => ownProps.customCone.iBeLoading === 'yep',
              isSuccess: ownProps => typeof ownProps.customCone.coneBabyCone === 'string'
            }
          ]
        }
      };
      expect(WithPageLoading.getDerivedStateFromProps(propsWithEverything)).toEqual({
        isLoading: true,
        isError: true,
        errorData: ['totally an error, brah.']
      });
    });
  });
  describe('renderLoading()', () => {
    it('should render a specific template if loading', () => {
      const spy = jest.spyOn(document, 'getElementById').mockImplementation(() => {
        const el = document.createElement('div');
        el.setAttribute('id', 'skel');
        el.setAttribute('data-skel-template', 'reviews');
        return el;
      });
      const wrapper = shallow(WithPageLoading.renderLoading());
      spy.mockRestore();
      expect(wrapper.find('[data-skel-template="reviews"]').exists()).toBe(true);
    });
    it('should render the empty skeleton while loading if a specific template was not specified', () => {
      const wrapper = shallow(WithPageLoading.renderLoading());
      expect(wrapper.find('[data-skel-template="empty"]').exists()).toBe(true);
    });
  });
  describe('renderError()', () => {
    it('should return Error400 if statusCode is 404 under networkError', () => {
      const instance = new WithPageLoading();
      instance.state = {
        errorData: [
          {
            networkError: {
              response: {
                status: 404
              }
            }
          }
        ]
      };
      // you need render when checking for text values. see docs on enzyme site.
      const wrapper = render(instance.renderError()).find('.maintext');
      expect(wrapper.text()).toBe('404 PAGE NOT FOUND');
    });
    it('should return Error400 if statusCode starts with 404 under graphQLErrors', () => {
      const instance = new WithPageLoading();
      instance.state = {
        errorData: [
          {
            graphQLErrors: [
              {
                message: '404 Dude. Duuuuude.'
              }
            ]
          }
        ]
      };
      // you need render when checking for text values. see docs on enzyme site.
      const wrapper = render(instance.renderError()).find('.maintext');
      expect(wrapper.text()).toBe('404 PAGE NOT FOUND');
    });
    it('should return Error500 if the gqlErrorMessage has a number besides 404', () => {
      const instance = new WithPageLoading();
      instance.state = {
        errorData: [
          {
            graphQLErrors: [
              {
                message: '798: what error is that?'
              }
            ]
          }
        ]
      };
      // you need render when checking for text values. see docs on enzyme site.
      const wrapper = render(instance.renderError()).find('.maintext');
      expect(wrapper.text()).toBe('INTERNAL SERVER ERROR');
    });
    it('should return Error500 if the gqlErrorMessage is gibberish', () => {
      const instance = new WithPageLoading();
      instance.state = {
        errorData: [
          {
            graphQLErrors: [
              {
                message: 'flibbedyflooooo!'
              }
            ]
          }
        ]
      };
      // you need render when checking for text values. see docs on enzyme site.
      const wrapper = render(instance.renderError()).find('.maintext');
      expect(wrapper.text()).toBe('INTERNAL SERVER ERROR');
    });
    it('should render 500 error by default', () => {
      const instance = new WithPageLoading();
      const wrapper = render(instance.renderError()).find('.maintext');
      expect(wrapper.text()).toBe('INTERNAL SERVER ERROR');
    });
  });
});

describe('withInventory integration tests', () => {
  it('should add a new property called withPageLoading to the target component of this HOC', () => {
    // these are props that apollo's graphql() hoc would add when a query is finished.
    const Component = () => <div>I am a component.</div>;
    const EnhancedComponent = withPageLoading()(Component);
    const wrapper = mount(<EnhancedComponent />);
    expect(wrapper.find('WithPageLoading').exists()).toBe(true);
    expect(typeof wrapper.find('Component').prop('withPageLoading')).not.toBe('undefined');
    expect(typeof wrapper.find('Component').prop('blahblorg')).toBe('undefined');
  });
  it('should render a loading state', () => {
    // these are props that apollo's graphql() hoc would add when a query is finished.
    const mockApolloQueryProps = {
      getFlavor: {
        loading: true,
        networkStatus: 1
      }
    };

    // config object for withPageLoading hoc
    const config = {
      isLoading: () => <div className="loading">I am Loading.</div>
    };

    const Component = () => <div>I am a component.</div>;
    const EnhancedComponent = compose(
      Decorated => ({ ...props }) => <Decorated {...props} {...mockApolloQueryProps} />,
      withPageLoading(config)
    )(Component);
    const wrapper = mount(<EnhancedComponent />);
    expect(wrapper.find('.skel').exists()).toBe(true);
  });
  it('should render a error state', () => {
    // these are props that apollo's graphql() hoc would add when a query is finished.
    const mockApolloQueryProps = {
      getFlavor: {
        error: 'I am a 500 error of some sort.',
        networkStatus: 8
      }
    };

    // config object for withPageLoading hoc
    const config = {
      isLoading: () => <div className="loading">I am Loading.</div>
    };

    const Component = () => <div>I am a component.</div>;
    const EnhancedComponent = compose(
      Decorated => ({ ...props }) => <Decorated {...props} {...mockApolloQueryProps} />,
      withPageLoading(config)
    )(Component);
    const wrapper = mount(<EnhancedComponent />);
    expect(wrapper.find('Error500').exists()).toBe(true);
  });
});
